"use strict";
Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "getPresetChain", {
  enumerable: true,
  get: function() {
    return getPresetChain;
  }
});
const _utils = require("@modern-js/utils");
const _babelchain = require("./babel-chain");
const es6BrowserList = [
  "chrome >= 61",
  "edge >= 16",
  "firefox >= 60",
  "safari >= 11",
  "ios_saf >= 11"
];
const getPresetOptions = (options) => typeof options === "object" ? options : {};
const getPresetChain = (option) => {
  const { appDirectory, presets: { envOptions, reactOptions, typescriptOptions } = {}, syntax = "es5", type = "module", runEnvironments = "browsers", jsxTransformRuntime = "automatic", useTsLoader = false, overrideBrowserslist } = option;
  const chain = (0, _babelchain.createBabelChain)();
  const disableEnvPreset = typeof envOptions === "boolean" && !envOptions;
  const disableReactPreset = typeof reactOptions === "boolean" && !reactOptions;
  const disableTypescriptPreset = typeof typescriptOptions === "boolean" && !typescriptOptions;
  if (!disableEnvPreset) {
    const browsersTargets = syntax === "es5" ? (0, _utils.getBrowserslist)(appDirectory) : es6BrowserList;
    const targets = runEnvironments === "node" ? {
      node: "12"
    } : browsersTargets;
    const presetEnvOptions = {
      targets: overrideBrowserslist || targets,
      modules: type === "commonjs" ? "commonjs" : false,
      bugfixes: runEnvironments !== "node",
      shippedProposals: type === "module" && syntax === "es6+",
      ...getPresetOptions(envOptions)
    };
    chain.preset("@babel/preset-env").use(require.resolve("@babel/preset-env"), [
      presetEnvOptions
    ]);
  }
  if (!disableReactPreset) {
    const classicOption = {
      useBuiltIns: !(type === "module" && syntax === "es5"),
      useSpread: type === "module" && syntax === "es5"
    };
    const automaticOption = {};
    const presetReactOptions = {
      runtime: jsxTransformRuntime === "classic" ? "classic" : "automatic",
      ...jsxTransformRuntime === "classic" ? classicOption : automaticOption,
      ...getPresetOptions(reactOptions)
    };
    chain.preset("@babel/preset-react").use(require.resolve("@babel/preset-react"), [
      presetReactOptions
    ]);
  }
  if (!(useTsLoader || disableTypescriptPreset)) {
    const typescriptPresetOptions = {
      ..._utils.DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS,
      ...getPresetOptions(typescriptOptions)
    };
    chain.preset("@babel/preset-typescript").use(require.resolve("@babel/preset-typescript"), [
      typescriptPresetOptions
    ]);
  }
  return chain;
};
